@@ -25,7 +25,7 @@ from six.moves import cPickle as pickle
 
 from docutils.io import NullOutput
 from docutils.core import Publisher
-from docutils.utils import Reporter, get_source_line
+from docutils.utils import Reporter, get_source_line, normalize_language_tag
 from docutils.utils.smartquotes import smartchars
 from docutils.parsers.rst import roles
 from docutils.parsers.rst.languages import en as english
@@ -35,11 +35,11 @@ from sphinx import addnodes
 from sphinx.io import SphinxStandaloneReader, SphinxDummyWriter, SphinxFileInput
 from sphinx.util import logging
 from sphinx.util import get_matching_docs, FilenameUniqDict, status_iterator
-from sphinx.util.nodes import WarningStream, is_translatable
+from sphinx.util.nodes import is_translatable
 from sphinx.util.osutil import SEP, ensuredir
 from sphinx.util.i18n import find_catalog_files
 from sphinx.util.console import bold  # type: ignore
-from sphinx.util.docutils import sphinx_domains
+from sphinx.util.docutils import sphinx_domains, WarningStream
 from sphinx.util.matching import compile_matchers
 from sphinx.util.parallel import ParallelTasks, parallel_available, make_chunks
 from sphinx.util.websupport import is_commentable
@@ -672,16 +672,22 @@ class BuildEnvironment(object):
         self.settings['trim_footnote_reference_space'] = \
             self.config.trim_footnote_reference_space
         self.settings['gettext_compact'] = self.config.gettext_compact
-        language = (self.config.language or 'en').replace('_', '-')
+
+        language = self.config.language or 'en'
         self.settings['language_code'] = language
+        self.settings['smart_quotes'] = True
         if self.config.html_use_smartypants is not None:
-            warnings.warn("html_use_smartypants option is deprecated. Use the "
-                          "smart_quotes option in docutils.conf instead.",
+            warnings.warn("html_use_smartypants option is deprecated. Smart "
+                          "quotes are on by default; if you want to disable "
+                          "or customize them, use the smart_quotes option in "
+                          "docutils.conf.",
                           RemovedInSphinx17Warning)
-            if language in smartchars.quotes:
-                self.settings['smart_quotes'] = self.config.html_use_smartypants
-        elif language in smartchars.quotes:  # We enable smartypants by default
-            self.settings['smart_quotes'] = True
+            self.settings['smart_quotes'] = self.config.html_use_smartypants
+        for tag in normalize_language_tag(language):
+            if tag in smartchars.quotes:
+                break
+        else:
+            self.settings['smart_quotes'] = False
 
         docutilsconf = path.join(self.srcdir, 'docutils.conf')
         # read docutils.conf from source dir, not from current dir
@@ -887,8 +893,7 @@ class BuildEnvironment(object):
         with open(doctree_filename, 'rb') as f:
             doctree = pickle.load(f)
         doctree.settings.env = self
-        doctree.reporter = Reporter(self.doc2path(docname), 2, 5,
-                                    stream=WarningStream(self._warnfunc))
+        doctree.reporter = Reporter(self.doc2path(docname), 2, 5, stream=WarningStream())
         return doctree
 
     def get_and_resolve_doctree(self, docname, builder, doctree=None,
@@ -943,10 +948,9 @@ class BuildEnvironment(object):
         try:
             # set env.docname during applying post-transforms
             self.temp_data['docname'] = docname
-            if hasattr(doctree, 'settings'):
-                doctree.settings.env = self
 
             transformer = SphinxTransformer(doctree)
+            transformer.set_environment(self)
             transformer.add_transforms(self.app.post_transforms)
             transformer.apply_transforms()
         finally:
