@@ -13,6 +13,7 @@ from docutils import nodes
 from docutils.transforms import Transform, Transformer
 from docutils.transforms.parts import ContentsFilter
 from docutils.utils import new_document
+from docutils.transforms.universal import SmartQuotes
 
 from sphinx import addnodes
 from sphinx.locale import _
@@ -85,6 +86,9 @@ class SphinxTransformer(Transformer):
     def apply_transforms(self):
         # type: () -> None
         if isinstance(self.document, nodes.document):
+            if not hasattr(self.document.settings, 'env') and self.env:
+                self.document.settings.env = self.env
+
             Transformer.apply_transforms(self)
         else:
             # wrap the target node by document node during transforming
@@ -324,3 +328,32 @@ class SphinxContentsFilter(ContentsFilter):
     def visit_image(self, node):
         # type: (nodes.Node) -> None
         raise nodes.SkipNode
+
+
+class SphinxSmartQuotes(SmartQuotes):
+    """
+    Customized SmartQuotes to avoid transform for some extra node types.
+
+    refs: sphinx.parsers.RSTParser
+    """
+    def get_tokens(self, txtnodes):
+        # A generator that yields ``(texttype, nodetext)`` tuples for a list
+        # of "Text" nodes (interface to ``smartquotes.educate_tokens()``).
+
+        texttype = {True: 'literal',  # "literal" text is not changed:
+                    False: 'plain'}
+        for txtnode in txtnodes:
+            nodetype = texttype[isinstance(txtnode.parent,
+                                           (nodes.literal,
+                                            nodes.literal_block,
+                                            addnodes.literal_emphasis,
+                                            addnodes.literal_strong,
+                                            addnodes.desc_signature,
+                                            addnodes.productionlist,
+                                            addnodes.desc_optional,
+                                            addnodes.desc_name,
+                                            nodes.math,
+                                            nodes.image,
+                                            nodes.raw,
+                                            nodes.problematic))]
+            yield (nodetype, txtnode.astext())
